@@ -1,3 +1,4 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
 /*
  * lib/krb5/krb/gen_seqnum.c
  *
@@ -8,7 +9,7 @@
  *   require a specific license from the United States Government.
  *   It is the responsibility of any person or organization contemplating
  *   export to obtain such a license before exporting.
- * 
+ *
  * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
  * distribute this software and its documentation for any purpose and
  * without fee is hereby granted, provided that the above copyright
@@ -22,7 +23,7 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- * 
+ *
  *
  * Routine to automatically generate a starting sequence number.
  * We do this by getting a random key and encrypting something with it,
@@ -35,21 +36,42 @@
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
 #endif
 
+static inline krb5_data
+key2data (krb5_keyblock k)
+{
+    krb5_data d;
+    d.magic = KV5M_DATA;
+    d.length = k.length;
+    d.data = (char *) k.contents;
+    return d;
+}
+
 krb5_error_code
-krb5_generate_seq_number(context, key, seqno)
-    krb5_context context;
-    krb5_const krb5_keyblock *key;
-    krb5_int32 *seqno;
+krb5_generate_seq_number(krb5_context context, const krb5_keyblock *key, krb5_ui_4 *seqno)
 {
     krb5_data seed;
     krb5_error_code retval;
 
-    seed.length = key->length;
-    seed.data = key->contents;
-    if ((retval = krb5_c_random_seed(context, &seed)))
-	return(retval);
+    seed = key2data(*key);
+    if ((retval = krb5_c_random_add_entropy(context, KRB5_C_RANDSOURCE_TRUSTEDPARTY, &seed)))
+        return(retval);
 
     seed.length = sizeof(*seqno);
     seed.data = (char *) seqno;
-    return(krb5_c_random_make_octets(context, &seed));
+    retval = krb5_c_random_make_octets(context, &seed);
+    if (retval)
+        return retval;
+    /*
+     * Work around implementation incompatibilities by not generating
+     * initial sequence numbers greater than 2^30.  Previous MIT
+     * implementations use signed sequence numbers, so initial
+     * sequence numbers 2^31 to 2^32-1 inclusive will be rejected.
+     * Letting the maximum initial sequence number be 2^30-1 allows
+     * for about 2^30 messages to be sent before wrapping into
+     * "negative" numbers.
+     */
+    *seqno &= 0x3fffffff;
+    if (*seqno == 0)
+        *seqno = 1;
+    return 0;
 }
