@@ -41,7 +41,7 @@
 
 /*! \brief Range for error codes
 
-    NetIDMgr errors range from \a KHM_ERROR_BASE to 
+    NetIDMgr errors range from \a KHM_ERROR_BASE to
     KHM_ERROR_BASE + KHM_ERROR_RANGE.
 */
 #define KHM_ERROR_RANGE 256L
@@ -168,6 +168,11 @@
 /*! \brief An incompatibility was found */
 #define KHM_ERROR_INCOMPATIBLE      (KHM_ERROR_BASE + 21)
 
+/*! \brief The operation was put on hold
+
+    A request was put on hold or postponed. */
+#define KHM_ERROR_HELD              (KHM_ERROR_BASE + 22)
+
 /*@}*/ /*kherror_codes*/
 
 /*! \brief Tests whether a return value indicates success */
