@@ -54,7 +54,7 @@
 
     If left undefined, the convenience macros will leave the facility
     value undefined.
- */ 
+ */
 #define KHERR_FACILITY NULL
 #endif
 
@@ -177,13 +177,13 @@ typedef struct tag_kherr_event {
     const wchar_t * suggestion; /*!< A suggested way to fix it
 			          (localized,formatted) */
 
-    kherr_severity   severity;  
+    kherr_severity   severity;
                                 /*!< Severity level.  One of the
 				  severity levels listed in
 				  enumeration ::kherr_severity */
     khm_int32   facility_id;    /*!< Left to the application to
 				  interpret */
-    kherr_suggestion suggestion_id; 
+    kherr_suggestion suggestion_id;
                                 /*!< One of the suggestion ID's from
 				  the enumeration
 				  ::kherr_suggestion */
@@ -217,44 +217,44 @@ typedef struct tag_kherr_event {
     is mutually exclusive.
  */
 enum kherr_event_flags {
-    KHERR_RF_CSTR_SHORT_DESC= 0x00000000, 
+    KHERR_RF_CSTR_SHORT_DESC= 0x00000000,
                                 /*!< Short description is a constant
                                   string */
-    KHERR_RF_RES_SHORT_DESC = 0x00000001, 
+    KHERR_RF_RES_SHORT_DESC = 0x00000001,
                                 /*!< Short description is a string
                                   resource */
-    KHERR_RF_MSG_SHORT_DESC = 0x00000002, 
+    KHERR_RF_MSG_SHORT_DESC = 0x00000002,
                                 /*!< Short description is a message
                                   resource */
-    KHERR_RF_FREE_SHORT_DESC= 0x00000004, 
+    KHERR_RF_FREE_SHORT_DESC= 0x00000004,
                                 /*!< Short description is an allocated
                                   string */
     KHERR_RFMASK_SHORT_DESC = 0x00000007,
 
-    KHERR_RF_CSTR_LONG_DESC = 0x00000000, 
+    KHERR_RF_CSTR_LONG_DESC = 0x00000000,
                                 /*!< Long description is a constant
                                   string */
-    KHERR_RF_RES_LONG_DESC  = 0x00000008, 
+    KHERR_RF_RES_LONG_DESC  = 0x00000008,
                                 /*!< Long description is a string
                                   resource */
-    KHERR_RF_MSG_LONG_DESC  = 0x00000010, 
+    KHERR_RF_MSG_LONG_DESC  = 0x00000010,
                                 /*!< Long description is a message
                                   resouce  */
-    KHERR_RF_FREE_LONG_DESC = 0x00000020, 
+    KHERR_RF_FREE_LONG_DESC = 0x00000020,
                                 /*!< Long description is an allocated
                                   string */
     KHERR_RFMASK_LONG_DESC  = 0x00000038,
 
-    KHERR_RF_CSTR_SUGGEST   = 0x00000000, 
+    KHERR_RF_CSTR_SUGGEST   = 0x00000000,
                                 /*!< Suggestion is a constant
                                   string */
-    KHERR_RF_RES_SUGGEST    = 0x00000040, 
+    KHERR_RF_RES_SUGGEST    = 0x00000040,
                                 /*!< Suggestion is a string
                                   resource */
-    KHERR_RF_MSG_SUGGEST    = 0x00000080, 
+    KHERR_RF_MSG_SUGGEST    = 0x00000080,
                                 /*!< Suggestion is a message
                                   resource */
-    KHERR_RF_FREE_SUGGEST   = 0x00000100,  
+    KHERR_RF_FREE_SUGGEST   = 0x00000100,
                                 /*!< Suggestion is an allocated
                                   string */
     KHERR_RFMASK_SUGGEST    = 0x000001C0,
@@ -298,7 +298,7 @@ typedef struct tag_kherr_context {
                                   number as well as the pointer to the
                                   context object. */
 
-    kherr_severity severity;    
+    kherr_severity severity;
 				/*!< Severity level.  One of the
 				  severity levels listed below. This
 				  is the severity level of the context
@@ -362,7 +362,7 @@ enum kherr_context_flags {
                                   threads whose current error context
                                   is this one. */
 
-    KHERR_CFMASK_INITIAL = 0x0000000a,
+    KHERR_CFMASK_INITIAL   = 0x0000000a,
                                 /*!< Allowed initial flags */
 };
 
@@ -379,13 +379,16 @@ enum kherr_context_flags {
     \see kherr_add_ctx_handler()
 */
 enum kherr_ctx_event {
-    KHERR_CTX_BEGIN  = 0x0001,  /*!< A new context was created */
-    KHERR_CTX_DESCRIBE=0x0002,  /*!< A context was described */
-    KHERR_CTX_END    = 0x0004,  /*!< A context was closed */
-    KHERR_CTX_ERROR  = 0x0008,  /*!< A context switched to an error
-                                  state */
-    KHERR_CTX_EVTCOMMIT = 0x0010 /*!< A event was committed into the
-				   context */
+    KHERR_CTX_BEGIN     = 0x00000001, /*!< A new context was created */
+    KHERR_CTX_DESCRIBE  = 0x00000002, /*!< A context was described */
+    KHERR_CTX_END       = 0x00000004, /*!< A context was closed */
+    KHERR_CTX_ERROR     = 0x00000008, /*!< A context switched to an
+                                        error state */
+    KHERR_CTX_EVTCOMMIT = 0x00000010, /*!< A event was committed into
+                                        the context */
+    KHERR_CTX_NEWCHILD  = 0x00000020, /*!< A new child context was created */
+    KHERR_CTX_FOLDCHILD = 0x00000040, /*!< A child context was folded */
+    KHERR_CTX_PROGRESS  = 0x00000080, /*!< Progress marker updated for context */
 };
 
 /*! \brief Context event handler
@@ -403,7 +406,7 @@ enum kherr_ctx_event {
 
     \see kherr_add_ctx_handler()
  */
-typedef void (KHMAPI * kherr_ctx_handler)(enum kherr_ctx_event, 
+typedef void (KHMAPI * kherr_ctx_handler)(enum kherr_ctx_event,
                                          kherr_context *);
 
 /*! \brief Add a context event handler
@@ -465,7 +468,7 @@ typedef void (KHMAPI * kherr_ctx_handler)(enum kherr_ctx_event,
         should be tracked.  If this is zero, all error contexts can
         trigger the handler.
  */
-KHMEXP void KHMAPI kherr_add_ctx_handler(kherr_ctx_handler h, 
+KHMEXP void KHMAPI kherr_add_ctx_handler(kherr_ctx_handler h,
                                          khm_int32 filter,
                                          kherr_serial serial);
 
@@ -577,14 +580,14 @@ KHMEXP void KHMAPI kherr_remove_ctx_handler(kherr_ctx_handler h,
         when the event is freed other than that implied by \a flags.
  */
 KHMEXP kherr_event * KHMAPI kherr_report(
-    enum kherr_severity severity,
+    kherr_severity severity,
     const wchar_t * short_desc,
     const wchar_t * facility,
     const wchar_t * location,
     const wchar_t * long_desC,
     const wchar_t * suggestion,
     khm_int32 facility_id,
-    enum kherr_suggestion suggestion_id,
+    kherr_suggestion suggestion_id,
     kherr_param p1,
     kherr_param p2,
     kherr_param p3,
@@ -603,7 +606,7 @@ KHMEXP kherr_event * KHMAPI kherr_report(
     string against the arguments.
  */
 KHMEXP kherr_event * __cdecl
-kherr_reportf_ex(enum kherr_severity severity,
+kherr_reportf_ex(kherr_severity severity,
                  const wchar_t * facility,
                  khm_int32 facility_id,
 #ifdef _WIN32
@@ -632,7 +635,7 @@ kherr_reportf(const wchar_t * long_desc_fmt,
     then only the first part of the string that fits within the limit
     is duplicated.
 
-    The resulign ::kherr_param must be passed in to kherr_report().
+    The resulting ::kherr_param must be passed in to kherr_report().
     The event logging framework will free the duplicated string once
     the data is no longer required.
  */
@@ -911,10 +914,39 @@ KHMEXP void KHMAPI kherr_set_progress(khm_ui_4 num, khm_ui_4 denom);
 #define _progress(num,denom) kherr_set_progress((num),(denom))
 
 /*! \brief Get the progress meter of the current error context
+
+    This is equivalent to calling kherr_get_progress_i() for the
+    current error context.  I.e. :
+
+    \code
+    kherr_context * ctx;
+
+    ctx = kherr_peek_context();
+    kherr_get_progress_i(ctx, &num, &denom);
+    kherr_release_context(ctx);
+    \endcode
+
+    \see kherr_get_progress_i()
  */
 KHMEXP void KHMAPI kherr_get_progress(khm_ui_4 * num, khm_ui_4 * denom);
 
 /*! \brief Get the progress meter of an error context
+
+    The progress meter for the current context can be set by calling
+    kherr_set_progress() (or using the ::_progress macro).  The
+    progress value returned by this function is as follows:
+
+    If one or more of the following conditions are true, then the
+    returned progress values are the values set for the context using
+    the most recent call to kherr_set_progress():
+
+    - if the numerator and the denominator are non-zero
+
+    - if the ::KHERR_CF_OWN_PROGRESS flag is set for the context.
+
+    Otherwise, the function will calculate the progress by enumerating
+    all the child context for the context and summing up the
+    normalized numerators and the denominators for them.
  */
 KHMEXP void KHMAPI kherr_get_progress_i(kherr_context * c, khm_ui_4 * num, khm_ui_4 * denom);
 
