@@ -104,6 +104,8 @@ void kmqint_free_msg_type(int t) {
 
     pt->completion_handler = NULL;
 
+    LDELETE(&all_msg_types, pt);
+
     PFREE(pt);
 }
 
@@ -130,7 +132,7 @@ void kmqint_msg_type_create(int t) {
     LeaveCriticalSection(&cs_kmq_types);
 }
 
-KHMEXP khm_int32 KHMAPI kmq_register_type(wchar_t * name, 
+KHMEXP khm_int32 KHMAPI kmq_register_type(wchar_t * name,
                                           khm_int32 * new_id)
 {
     int i;
@@ -152,7 +154,7 @@ KHMEXP khm_int32 KHMAPI kmq_register_type(wchar_t * name,
             /* continue searching since we might find that this type
                is already registered. */
         } else {
-            if(msg_types[i]->name != NULL && 
+            if(msg_types[i]->name != NULL &&
                !wcscmp(msg_types[i]->name, name)) {
 
                 registered = TRUE;
@@ -316,8 +318,8 @@ kmq_msg_subscription * kmqint_msg_type_del_sub_cb(khm_int32 t, kmq_callback_t cb
     s = msg_types[t]->subs;
     while(s) {
         kmq_msg_subscription * n = LNEXT(s);
-        if(s->rcpt_type == KMQ_RCPTTYPE_CB && 
-           s->recipient.cb == cb && 
+        if(s->rcpt_type == KMQ_RCPTTYPE_CB &&
+           s->recipient.cb == cb &&
            s->queue == q) {
             /*TODO: do more here? */
             LDELETE(&msg_types[t]->subs, s);
@@ -374,8 +376,12 @@ khm_int32 kmqint_msg_type_set_handler(khm_int32 type, kmq_msg_completion_handler
     if (type == KMSG_SYSTEM)
         return KHM_ERROR_INVALID_PARAM;
 
-    if(!msg_types[type])
-        kmqint_msg_type_create(type);
+    if(!msg_types[type]) {
+        if (handler)
+            kmqint_msg_type_create(type);
+        else
+            return KHM_ERROR_SUCCESS;
+    }
 
     if(!msg_types[type])
         return KHM_ERROR_NO_RESOURCES;
