@@ -75,7 +75,7 @@ ATOM khui_htwnd_cls;
 struct tx_tbl_t {
     wchar_t *   string;
     LONG        value;
-} 
+}
 
 htw_color_table[] = {
     {L"black", RGB(0,0,0)},
@@ -278,7 +278,7 @@ static wchar_t * skip_ws(wchar_t * s) {
     return s;
 }
 
-/* s points to something like " = \"value\"" 
+/* s points to something like " = \"value\""
    start and len will point to the start and
    length of value.  return value will point to the
    character following the last double quote. */
@@ -332,14 +332,14 @@ We currently support the following tags:
 */
 
 static int htw_parse_tag(
-    wchar_t * start, 
-    wchar_t ** end, 
-    int * align, 
-    khui_htwnd_data * d, 
-    format_stack * s, 
+    wchar_t * start,
+    wchar_t ** end,
+    int * align,
+    khui_htwnd_data * d,
+    format_stack * s,
     PPOINT p_abs,
     PPOINT p_rel,
-    int lh, 
+    int lh,
     BOOL dry_run)
 {
     wchar_t * c;
@@ -454,7 +454,7 @@ static int htw_parse_tag(
         c = wcschr(c, L'>');
         if(!c)
             c = c + wcslen(c);
-        
+
 
         if(align_s)
             *align = table_lookup(htw_align_table, ARRAYLENGTH(htw_align_table), align_s, align_len);
@@ -463,23 +463,23 @@ static int htw_parse_tag(
 
         n = 1;
     } else if(!_wcsnicmp(start, L"b", c - start)) {
-        format_push(s,d, HTW_DEFAULT, FV_BOLD, HTW_DEFAULT);
+        format_push(s,d, HTW_DEFAULT, FV_BOLD, (COLORREF) HTW_DEFAULT);
     } else if(!_wcsnicmp(start, L"/b", c - start)) {
         format_pop(s);
     } else if(!_wcsnicmp(start, L"u", c - start)) {
-        format_push(s,d, HTW_DEFAULT, FV_UNDERLINE, HTW_DEFAULT);
+        format_push(s,d, HTW_DEFAULT, FV_UNDERLINE, (COLORREF) HTW_DEFAULT);
     } else if(!_wcsnicmp(start, L"/u", c - start)) {
         format_pop(s);
     } else if(!_wcsnicmp(start, L"i", c - start)) {
-        format_push(s,d, HTW_DEFAULT, FV_ITALIC, HTW_DEFAULT);
+        format_push(s,d, HTW_DEFAULT, FV_ITALIC, (COLORREF) HTW_DEFAULT);
     } else if(!_wcsnicmp(start, L"/i", c - start)) {
         format_pop(s);
     } else if(!_wcsnicmp(start, L"large", c - start)) {
-        format_push(s,d,-MulDiv(HTW_LARGE_SIZE, d->l_pixel_y, 72), HTW_DEFAULT, HTW_DEFAULT);
+        format_push(s,d,-MulDiv(HTW_LARGE_SIZE, d->l_pixel_y, 72), HTW_DEFAULT, (COLORREF) HTW_DEFAULT);
     } else if(!_wcsnicmp(start, L"/large", c - start)) {
         format_pop(s);
     } else if(!_wcsnicmp(start, L"huge", c - start)) {
-        format_push(s,d,-MulDiv(HTW_HUGE_SIZE, d->l_pixel_y, 72), HTW_DEFAULT, HTW_DEFAULT);
+        format_push(s,d,-MulDiv(HTW_HUGE_SIZE, d->l_pixel_y, 72), HTW_DEFAULT, (COLORREF) HTW_DEFAULT);
     } else if(!_wcsnicmp(start, L"/huge", c - start)) {
         format_pop(s);
     } else if(!_wcsnicmp(start, L"center", c - start)) {
@@ -489,7 +489,7 @@ static int htw_parse_tag(
         *align = ALIGN_CENTER;
         n = 1;
     } else if(!_wcsnicmp(start, L"left", c - start) ||
-        !_wcsnicmp(start, L"p", c - start)) 
+        !_wcsnicmp(start, L"p", c - start))
     {
         c = wcschr(c, L'>');
         if(!c)
@@ -610,9 +610,9 @@ static void htw_assert_style(HDC hdc, khui_htwnd_data * d, int style)
     lf.lfEscapement = 0;
     lf.lfOrientation = 0;
     lf.lfWeight = (d->styles[style].variation & FV_BOLD)? FW_BOLD: FW_NORMAL;
-    lf.lfItalic = !!(d->styles[style].variation & FV_ITALIC);
-    lf.lfUnderline = !!(d->styles[style].variation & FV_UNDERLINE);
-    lf.lfStrikeOut = !!(d->styles[style].variation & FV_STRIKEOUT);
+    lf.lfItalic = (BYTE) !!(d->styles[style].variation & FV_ITALIC);
+    lf.lfUnderline = (BYTE) !!(d->styles[style].variation & FV_UNDERLINE);
+    lf.lfStrikeOut = (BYTE) !!(d->styles[style].variation & FV_STRIKEOUT);
     lf.lfCharSet = DEFAULT_CHARSET;
     lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
     lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
@@ -642,6 +642,8 @@ static LRESULT htw_paint(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
     int ext_height = 0;
 
     d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+    if (d == NULL)
+        return 0;
 
     if(!GetUpdateRect(hwnd, &r, !(d->flags & KHUI_HTWND_TRANSPARENT)))
         return 0;
@@ -838,7 +840,7 @@ static LRESULT htw_paint(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
             else
                 d->flags &= ~KHUI_HTWND_VSCROLL;
 
-            l = GetWindowLongPtr(hwnd, GWL_STYLE);
+            l = (LONG) GetWindowLongPtr(hwnd, GWL_STYLE);
             l &= ~(WS_HSCROLL | WS_VSCROLL);
 
             l |= ((d->flags & KHUI_HTWND_HSCROLL) ? WS_HSCROLL : 0) |
@@ -945,6 +947,9 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             BOOL rv;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                return 0;
+
             newtext = (wchar_t *) lParam;
 
             if(d->text) {
@@ -978,21 +983,24 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             int i;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
-            if(d->text)
-                PFREE(d->text);
-            d->text = 0;
+            if (d) {
+                if(d->text)
+                    PFREE(d->text);
+                d->text = 0;
 
-            if(d->links) {
-                for(i=0;i<d->max_links;i++) {
-                    if(d->links[i])
-                        PFREE(d->links[i]);
+                if(d->links) {
+                    for(i=0;i<d->max_links;i++) {
+                        if(d->links[i])
+                            PFREE(d->links[i]);
+                    }
+                    PFREE(d->links);
                 }
-                PFREE(d->links);
+
+                clear_styles(d);
+
+                PFREE(d);
+                SetWindowLongPtr(hwnd, 0, 0);
             }
-
-            clear_styles(d);
-
-            PFREE(d);
         }
         break;
 
@@ -1004,6 +1012,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             RECT r;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                return FALSE;
 
             GetClientRect(hwnd, &r);
             hbr = GetSysColorBrush(COLOR_WINDOW);
@@ -1038,8 +1048,10 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
 
             if(hwnd != (HWND)wParam)
                 break;
-                
+
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             if(d->active_link >= 0) {
                 SetCursor(d->hc_hand);
@@ -1053,6 +1065,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             khui_htwnd_data * d;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             d->flags |= KHUI_HTWND_FOCUS;
 
@@ -1065,6 +1079,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             khui_htwnd_data * d;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             d->flags &= ~KHUI_HTWND_FOCUS;
 
@@ -1077,6 +1093,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             khui_htwnd_data * d;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             d->md_link = d->active_link;
 
@@ -1089,6 +1107,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             khui_htwnd_data * d;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             if(d->md_link == d->active_link && d->md_link >= 0) {
                 /* clicked */
@@ -1109,6 +1129,8 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             RECT r;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
 
             old_pos = new_pos = d->scroll_left;
             ext = d->ext_width;
@@ -1194,9 +1216,12 @@ LRESULT CALLBACK khui_htwnd_proc(HWND hwnd,
             int nl;
 
             d = (khui_htwnd_data *)(LONG_PTR) GetWindowLongPtr(hwnd, 0);
+            if (d == NULL)
+                break;
+
             p.x = GET_X_LPARAM(lParam) + d->scroll_left;
             p.y = GET_Y_LPARAM(lParam) + d->scroll_top;
-                
+
             for(i=0; i<d->n_links; i++) {
                 if(d->links && d->links[i] && PtInRect(&(d->links[i]->r), p))
                     break;
